'***********************************************************************
'
'    Filename        :     Kchentimer		
'    File Version    :     1.0
'    Datum           :     24.07.2009             	
'    Autor           :     Peter Verderber		
'    Funktion        :     Kurzzeitmesser	
'    Letzte Revision :     1.0
'    Datum           :     24.07.2009          
'    Ziel PIC        :     16F628A	
'    Compiler        :     iL_BAS16 - Professional V6.0-3.4  07.07.2009
'    
'***********************************************************************
'*****                Definitionen und Konstanten                  *****
'***********************************************************************
  
Date
Time

$LIST INHX8    'erzeugt eine OBJ-Datei im Intel-Hex8-Format (default)
'$LIST OBJ2HEX  'das Hexfile bekommt anstatt OBJ die Dateiendung HEX

$NCALDEF
         
'hier wird der Prozessortyp und dessen Spezifikationen definiert

define device 16F628A

config wdt_off
config irc_osc
config pwrte_on
config t0se1
config psa1 
config mclr_ext
config cmcfg7       
                 
'xtal 4.00
        
tris ra,%00001100
tris rb,%10000000

output rb,0
                  
'$DEBUG 3,ra,4
 
define hh            = auto as byte      'Uhrzeit Stunden
define mm            = auto as byte      'Uhrzeit Minuten 
define ss            = auto as byte      'Uhrzeit Sekunden
define ahh           = auto as byte      'Alarmzeit Stunden          
define amm           = auto as byte      'Alarmzeit Minuten  

define rtc_mm_einer  = auto as byte
define rtc_mm_zehner = auto as byte
define rtc_mm_dez    = auto as byte
define rtc_hh_einer  = auto as byte
define rtc_hh_zehner = auto as byte  
define rtc_hh_dez    = auto as byte

define flag          = auto as byte
define hh_flag       = auto as byte
define mm_flag       = auto as byte

define rtc_ss        = auto as byte      'Sekunden high-nibble=10er, low-nibble=1er
define rtc_mm        = auto as byte      'Minuten  high-nibble=10er, low-nibble=1er
define rtc_hh        = auto as byte      'Stunden  high-nibble=10er, low-nibble=1er
define rtc_xx        = auto as byte

define sek_einer     = auto as byte      '0. BCD-Byte
define sek_zehner    = auto as byte      '1. BCD-Byte
define min_einer     = auto as byte      '2. BCD-Byte
define min_zehner    = auto as byte      '3. BCD-Byte
define std_einer     = auto as byte      '4. BCD-Byte
define std_zehner    = auto as byte      '5. BCD-Byte

define temp0         = auto as byte      'Hilfsvariablen
define temp1         = auto as byte

define ue            = $F5 as const      'Anzeige  auf LCD-Display


define plus_taste    = ra,2         'Taste Zeit inkrementieren
define minus_taste   = ra,3         'Taste Zeit dekrementieren 
define start_taste   = rb,7         'Taste START/STOP/Alarmzeit setzen
define beeper        = rb,6         'Summer           

goto main                         'Start Hauptprogramm

'***********************************************************************
'*****                   Unterprogramme                            *****
'***********************************************************************

init:
  i2cinit ra,1,0                       'IC-Initialisierung 
                                       'SDA = ra,1 SCL = ra,0
         
  wait 1000                            'LCD-Initialisierung 
  lcdinit rb,2,16
              '"1234567890123456"                        
  lcdwrite 1,1,"  K chen-Timer  "      'Begrungstext
  lcdwrite 1,4,ue 
  lcdwrite 2,1,"PVC - electronic"
  wait 2500
  return

'------------------------------------------------------------------------

init_var:
  let mm      = 0
  let hh      = 0
  let ss      = 0
  let flag    = 0
  let hh_flag = 0
  let mm_flag = 0
  
  let rtc_mm_dez    = 0
  let rtc_mm_zehner = 0
  let rtc_mm_einer  = 0
  let rtc_hh_dez    = 0
  let rtc_hh_zehner = 0
  let rtc_hh_einer  = 0
  return

'-----------------------------------------------------------------------  

init_DS1307:                           'RTC_DS1307-Initialisierung
  
  i2cwrite %11010000,$00,%00000000     'Sekunden
  i2cwrite %11010000,$01,%00000000     'Minuten
  i2cwrite %11010000,$02,%00000000     'Stunden       
  i2cwrite %11010000,$03,%00000001     'Wochentag
  i2cwrite %11010000,$04,%00000001     'Tag
  i2cwrite %11010000,$05,%00000001     'Monat
  i2cwrite %11010000,$06,%00001001     'Jahr
  i2cwrite %11010001,$07,%00010000     '1Hz-Impulse
           
  return
  
'-----------------------------------------------------------------------

read_DS1307:                           'DS1307-Lesen
         
  i2cwrite %11010000,$00  
  i2cread  %11010001,rtc_ss            'Sekunden          
  i2cread  %11010001,rtc_mm            'Minuten
  i2cread  %11010001,rtc_hh            'Stunden
  i2cread  %11010000,rtc_xx            'Wochentag
  i2cread  %11010000,rtc_xx            'Tag
  i2cread  %11010000,rtc_xx            'Monat
  i2cread  %11010000,rtc_xx            'Jahr
  
  return
    
'-----------------------------------------------------------------------  

zeit_anzeige:     'Uhrzeit fr die Ausgabe auf LCD vorbereiten
  
  gosub read_DS1307                    'rtc lesen
                                       'sekunden 1x and x1
  let temp0 = rtc_ss and %00001111     'Werte von 0 bis 9
  let temp1 = rtc_ss
  let sek_einer = temp0 + 48           'nun ist es ein ascii-Zeichen
  swap temp1
  let temp0 = temp1 and %00000111      'Werte von 0 bis 5
  let sek_zehner = temp0 + 48          'nun ist es ein ascii-Zeichen  
         
                                       'minuten 1x and x1        
  let temp0 = rtc_mm and %00001111     'Werte von 0 bis 9
  let temp1 = rtc_mm
  let rtc_mm_einer = temp0
  let min_einer = temp0 + 48           'nun ist es ein ascii-Zeichen
  swap temp1
  let temp0 = temp1 and %00000111      'Werte von 0 bis 5
  let rtc_mm_zehner = temp0 * 10         
  let min_zehner = temp0 + 48          'nun ist es ein ascii-Zeichen
  
                                       'stunden 1x and x1         
  let temp0 = rtc_hh and %00001111     'Werte von 0 bis 9
  let temp1 = rtc_hh
  let rtc_hh_einer = temp0
  let std_einer = temp0 + 48           'nun ist es ein ascii-Zeichen
  swap temp1
  let temp0 = temp1 and %00000011      'Werte von 0 bis 2
  let rtc_hh_zehner = temp0 * 10
  let std_zehner = temp0 + 48          'nun ist es ein ascii-Zeichen
    
  lcdwrite 2,4,"IST  ",std_zehner,#,std_einer,#,":",min_zehner,#,{+}
           min_einer,#,":",sek_zehner,#,sek_einer,#
  
  return   
  
'------------------------------------------------------------------------
  
alarm_anzeige:     'Alarmzeit fr die Ausgabe auf LCD vorbereiten
      
  lcdchar  0,$0E,$15,$15,$17,$11,$11,$0E,$00
  lcdwrite 1,2,0
  lcdwrite 1,4,"SOLL 00:00"               
  lcdwrite 1,8,ahh
  lcdwrite 1,11,amm,":00"
  if amm < 10 then lcdwrite 1,11,":0" else lcdwrite 1,11,":"
  if ahh < 10 then lcdwrite 1,9,"0"
     
  return   
   
'------------------------------------------------------------------------  

set_alarm:
              '"1234567890123456"                        
  lcdwrite 1,1,"Alarm EINSTELLEN"      
  
  lcdwrite 2,1,"SET Stunden     "
  lofreq beeper,1000,100
  set_ahh:
    if plus_taste = 0 then inc ahh 
    if ahh > 23 then let ahh = 0
    
    if minus_taste = 0 then dec ahh
    if ahh = 0 then let ahh = 0
    
    lcdwrite 2,13,ahh
    wait 200
    if start_taste = 1 then goto set_ahh 
    
    
    
  lcdwrite 2,1,"SET Minuten     "
  lofreq beeper,1000,100   
  set_amm:
    if plus_taste = 0 then inc amm 
    if amm > 59 then let amm = 0
    
    if minus_taste = 0 then dec amm
    if amm = 0 then let amm = 0
    
    lcdwrite 2,13,amm
    wait 200
    if start_taste = 1 then goto set_amm 

  gosub store_alarm
  
  lcdwrite 2,1,"Zeit GESPEICHERT"
  lofreq beeper,1000,100
  wait 1000
  lcdwrite 2,1," Timer  STARTEN "
  lofreq beeper,1000,100   
  return
  
'------------------------------------------------------------------------
  
store_alarm:                      
  write 00, ahh              'Speichern Alarmzeit Stunden
  wait 15
  write 01, amm              'Speichern Alarmzeit Minuten
  wait 15
  
  return

'------------------------------------------------------------------------
   
read_alarm:                      
  read 00, ahh               'Lesen Alarmzeit Stunden
  wait 15
  read 01, amm               'Lesen Alarmzeit Minuten
  wait 15
  
  return     

'------------------------------------------------------------------------
    

alarm:                       'Alarmton
  lofreq beeper,1500,100	
    
  wait 70
  
  if start_taste = 0 then goto main 
  
  goto alarm

'***********************************************************************
'*****                     Hauptprogramm                           *****
'***********************************************************************

main:
  gosub init                 'LCD-Initialisierung
  gosub init_DS1307          'RTC_initielisierung
  gosub init_var             'Variablen-Initialisierung
  gosub read_alarm           'letzte Alarmzeit einlesen          
  gosub set_alarm            'Alarmzeit einstellen
  
  loop:
    if start_taste = 1 then goto loop   
    
    gosub init_var           'Variablen-Initialisierung
    gosub init_DS1307        'RTC_initielisierung
    lcdclear
    lofreq beeper,1000,100
    
    uhr:
      gosub zeit_anzeige
      gosub alarm_anzeige   
      
      let rtc_mm_dez = rtc_mm_zehner + rtc_mm_einer
      let rtc_hh_dez = rtc_hh_zehner + rtc_hh_einer
      
      if rtc_hh_dez  = ahh then let hh_flag = 1  
      if rtc_mm_dez  = amm then let mm_flag = 1
  
      let flag = hh_flag + mm_flag 
      lcdchar 1,$01,$0B,$0D,$19,$19,$0D,$0B,$01
      if flag = 2 then lcdwrite 2,2,1
      if flag = 2 then goto alarm   
    
      goto uhr  
 
  goto main
        
  end                                  'Ende gut, alles gut       
        
        